# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, cast

import httpx

from ..._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ..._utils import path_template, maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._wrappers import ResultWrapper
from ...pagination import SyncV4PagePaginationArray, AsyncV4PagePaginationArray
from ..._base_client import AsyncPaginator, make_request_options
from ...types.ai_gateway import custom_provider_list_params, custom_provider_create_params
from ...types.ai_gateway.custom_provider_get_response import CustomProviderGetResponse
from ...types.ai_gateway.custom_provider_list_response import CustomProviderListResponse
from ...types.ai_gateway.custom_provider_create_response import CustomProviderCreateResponse
from ...types.ai_gateway.custom_provider_delete_response import CustomProviderDeleteResponse

__all__ = ["CustomProvidersResource", "AsyncCustomProvidersResource"]


class CustomProvidersResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> CustomProvidersResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return CustomProvidersResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> CustomProvidersResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return CustomProvidersResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        account_id: str,
        base_url: str,
        name: str,
        slug: str,
        beta: bool | Omit = omit,
        curl_example: str | Omit = omit,
        description: str | Omit = omit,
        enable: bool | Omit = omit,
        headers: str | Omit = omit,
        js_example: str | Omit = omit,
        link: str | Omit = omit,
        position: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CustomProviderCreateResponse:
        """
        Creates a new AI Gateway.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._post(
            path_template("/accounts/{account_id}/ai-gateway/custom-providers", account_id=account_id),
            body=maybe_transform(
                {
                    "base_url": base_url,
                    "name": name,
                    "slug": slug,
                    "beta": beta,
                    "curl_example": curl_example,
                    "description": description,
                    "enable": enable,
                    "headers": headers,
                    "js_example": js_example,
                    "link": link,
                    "position": position,
                },
                custom_provider_create_params.CustomProviderCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[CustomProviderCreateResponse]._unwrapper,
            ),
            cast_to=cast(Type[CustomProviderCreateResponse], ResultWrapper[CustomProviderCreateResponse]),
        )

    def list(
        self,
        *,
        account_id: str,
        beta: bool | Omit = omit,
        enable: bool | Omit = omit,
        page: int | Omit = omit,
        per_page: int | Omit = omit,
        search: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncV4PagePaginationArray[CustomProviderListResponse]:
        """
        Lists all AI Gateway evaluator types configured for the account.

        Args:
          search: Search by id, name, slug

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/ai-gateway/custom-providers", account_id=account_id),
            page=SyncV4PagePaginationArray[CustomProviderListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "beta": beta,
                        "enable": enable,
                        "page": page,
                        "per_page": per_page,
                        "search": search,
                    },
                    custom_provider_list_params.CustomProviderListParams,
                ),
            ),
            model=CustomProviderListResponse,
        )

    def delete(
        self,
        id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CustomProviderDeleteResponse:
        """
        Deletes an AI Gateway dataset.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._delete(
            path_template("/accounts/{account_id}/ai-gateway/custom-providers/{id}", account_id=account_id, id=id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[CustomProviderDeleteResponse]._unwrapper,
            ),
            cast_to=cast(Type[CustomProviderDeleteResponse], ResultWrapper[CustomProviderDeleteResponse]),
        )

    def get(
        self,
        id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CustomProviderGetResponse:
        """
        Retrieves details for a specific AI Gateway dataset.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            path_template("/accounts/{account_id}/ai-gateway/custom-providers/{id}", account_id=account_id, id=id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[CustomProviderGetResponse]._unwrapper,
            ),
            cast_to=cast(Type[CustomProviderGetResponse], ResultWrapper[CustomProviderGetResponse]),
        )


class AsyncCustomProvidersResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncCustomProvidersResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncCustomProvidersResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncCustomProvidersResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncCustomProvidersResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        account_id: str,
        base_url: str,
        name: str,
        slug: str,
        beta: bool | Omit = omit,
        curl_example: str | Omit = omit,
        description: str | Omit = omit,
        enable: bool | Omit = omit,
        headers: str | Omit = omit,
        js_example: str | Omit = omit,
        link: str | Omit = omit,
        position: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CustomProviderCreateResponse:
        """
        Creates a new AI Gateway.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._post(
            path_template("/accounts/{account_id}/ai-gateway/custom-providers", account_id=account_id),
            body=await async_maybe_transform(
                {
                    "base_url": base_url,
                    "name": name,
                    "slug": slug,
                    "beta": beta,
                    "curl_example": curl_example,
                    "description": description,
                    "enable": enable,
                    "headers": headers,
                    "js_example": js_example,
                    "link": link,
                    "position": position,
                },
                custom_provider_create_params.CustomProviderCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[CustomProviderCreateResponse]._unwrapper,
            ),
            cast_to=cast(Type[CustomProviderCreateResponse], ResultWrapper[CustomProviderCreateResponse]),
        )

    def list(
        self,
        *,
        account_id: str,
        beta: bool | Omit = omit,
        enable: bool | Omit = omit,
        page: int | Omit = omit,
        per_page: int | Omit = omit,
        search: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[CustomProviderListResponse, AsyncV4PagePaginationArray[CustomProviderListResponse]]:
        """
        Lists all AI Gateway evaluator types configured for the account.

        Args:
          search: Search by id, name, slug

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/ai-gateway/custom-providers", account_id=account_id),
            page=AsyncV4PagePaginationArray[CustomProviderListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "beta": beta,
                        "enable": enable,
                        "page": page,
                        "per_page": per_page,
                        "search": search,
                    },
                    custom_provider_list_params.CustomProviderListParams,
                ),
            ),
            model=CustomProviderListResponse,
        )

    async def delete(
        self,
        id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CustomProviderDeleteResponse:
        """
        Deletes an AI Gateway dataset.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._delete(
            path_template("/accounts/{account_id}/ai-gateway/custom-providers/{id}", account_id=account_id, id=id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[CustomProviderDeleteResponse]._unwrapper,
            ),
            cast_to=cast(Type[CustomProviderDeleteResponse], ResultWrapper[CustomProviderDeleteResponse]),
        )

    async def get(
        self,
        id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CustomProviderGetResponse:
        """
        Retrieves details for a specific AI Gateway dataset.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            path_template("/accounts/{account_id}/ai-gateway/custom-providers/{id}", account_id=account_id, id=id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[CustomProviderGetResponse]._unwrapper,
            ),
            cast_to=cast(Type[CustomProviderGetResponse], ResultWrapper[CustomProviderGetResponse]),
        )


class CustomProvidersResourceWithRawResponse:
    def __init__(self, custom_providers: CustomProvidersResource) -> None:
        self._custom_providers = custom_providers

        self.create = to_raw_response_wrapper(
            custom_providers.create,
        )
        self.list = to_raw_response_wrapper(
            custom_providers.list,
        )
        self.delete = to_raw_response_wrapper(
            custom_providers.delete,
        )
        self.get = to_raw_response_wrapper(
            custom_providers.get,
        )


class AsyncCustomProvidersResourceWithRawResponse:
    def __init__(self, custom_providers: AsyncCustomProvidersResource) -> None:
        self._custom_providers = custom_providers

        self.create = async_to_raw_response_wrapper(
            custom_providers.create,
        )
        self.list = async_to_raw_response_wrapper(
            custom_providers.list,
        )
        self.delete = async_to_raw_response_wrapper(
            custom_providers.delete,
        )
        self.get = async_to_raw_response_wrapper(
            custom_providers.get,
        )


class CustomProvidersResourceWithStreamingResponse:
    def __init__(self, custom_providers: CustomProvidersResource) -> None:
        self._custom_providers = custom_providers

        self.create = to_streamed_response_wrapper(
            custom_providers.create,
        )
        self.list = to_streamed_response_wrapper(
            custom_providers.list,
        )
        self.delete = to_streamed_response_wrapper(
            custom_providers.delete,
        )
        self.get = to_streamed_response_wrapper(
            custom_providers.get,
        )


class AsyncCustomProvidersResourceWithStreamingResponse:
    def __init__(self, custom_providers: AsyncCustomProvidersResource) -> None:
        self._custom_providers = custom_providers

        self.create = async_to_streamed_response_wrapper(
            custom_providers.create,
        )
        self.list = async_to_streamed_response_wrapper(
            custom_providers.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            custom_providers.delete,
        )
        self.get = async_to_streamed_response_wrapper(
            custom_providers.get,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, cast

import httpx

from ..._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ..._utils import path_template, maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._wrappers import ResultWrapper
from ...pagination import SyncV4PagePaginationArray, AsyncV4PagePaginationArray
from ..._base_client import AsyncPaginator, make_request_options
from ...types.workflows import version_list_params
from ...types.workflows.version_get_response import VersionGetResponse
from ...types.workflows.version_list_response import VersionListResponse
from ...types.workflows.version_graph_response import VersionGraphResponse

__all__ = ["VersionsResource", "AsyncVersionsResource"]


class VersionsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> VersionsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return VersionsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> VersionsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return VersionsResourceWithStreamingResponse(self)

    def list(
        self,
        workflow_name: str,
        *,
        account_id: str,
        page: float | Omit = omit,
        per_page: float | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncV4PagePaginationArray[VersionListResponse]:
        """
        Lists all deployed versions of a workflow.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not workflow_name:
            raise ValueError(f"Expected a non-empty value for `workflow_name` but received {workflow_name!r}")
        return self._get_api_list(
            path_template(
                "/accounts/{account_id}/workflows/{workflow_name}/versions",
                account_id=account_id,
                workflow_name=workflow_name,
            ),
            page=SyncV4PagePaginationArray[VersionListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "page": page,
                        "per_page": per_page,
                    },
                    version_list_params.VersionListParams,
                ),
            ),
            model=VersionListResponse,
        )

    def get(
        self,
        version_id: str,
        *,
        account_id: str,
        workflow_name: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> VersionGetResponse:
        """
        Retrieves details for a specific deployed workflow version.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not workflow_name:
            raise ValueError(f"Expected a non-empty value for `workflow_name` but received {workflow_name!r}")
        if not version_id:
            raise ValueError(f"Expected a non-empty value for `version_id` but received {version_id!r}")
        return self._get(
            path_template(
                "/accounts/{account_id}/workflows/{workflow_name}/versions/{version_id}",
                account_id=account_id,
                workflow_name=workflow_name,
                version_id=version_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[VersionGetResponse]._unwrapper,
            ),
            cast_to=cast(Type[VersionGetResponse], ResultWrapper[VersionGetResponse]),
        )

    def graph(
        self,
        version_id: str,
        *,
        account_id: str,
        workflow_name: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> VersionGraphResponse:
        """
        Retrieves the graph visualization of a workflow version.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not workflow_name:
            raise ValueError(f"Expected a non-empty value for `workflow_name` but received {workflow_name!r}")
        if not version_id:
            raise ValueError(f"Expected a non-empty value for `version_id` but received {version_id!r}")
        return self._get(
            path_template(
                "/accounts/{account_id}/workflows/{workflow_name}/versions/{version_id}/graph",
                account_id=account_id,
                workflow_name=workflow_name,
                version_id=version_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[VersionGraphResponse]._unwrapper,
            ),
            cast_to=cast(Type[VersionGraphResponse], ResultWrapper[VersionGraphResponse]),
        )


class AsyncVersionsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncVersionsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncVersionsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncVersionsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncVersionsResourceWithStreamingResponse(self)

    def list(
        self,
        workflow_name: str,
        *,
        account_id: str,
        page: float | Omit = omit,
        per_page: float | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[VersionListResponse, AsyncV4PagePaginationArray[VersionListResponse]]:
        """
        Lists all deployed versions of a workflow.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not workflow_name:
            raise ValueError(f"Expected a non-empty value for `workflow_name` but received {workflow_name!r}")
        return self._get_api_list(
            path_template(
                "/accounts/{account_id}/workflows/{workflow_name}/versions",
                account_id=account_id,
                workflow_name=workflow_name,
            ),
            page=AsyncV4PagePaginationArray[VersionListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "page": page,
                        "per_page": per_page,
                    },
                    version_list_params.VersionListParams,
                ),
            ),
            model=VersionListResponse,
        )

    async def get(
        self,
        version_id: str,
        *,
        account_id: str,
        workflow_name: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> VersionGetResponse:
        """
        Retrieves details for a specific deployed workflow version.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not workflow_name:
            raise ValueError(f"Expected a non-empty value for `workflow_name` but received {workflow_name!r}")
        if not version_id:
            raise ValueError(f"Expected a non-empty value for `version_id` but received {version_id!r}")
        return await self._get(
            path_template(
                "/accounts/{account_id}/workflows/{workflow_name}/versions/{version_id}",
                account_id=account_id,
                workflow_name=workflow_name,
                version_id=version_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[VersionGetResponse]._unwrapper,
            ),
            cast_to=cast(Type[VersionGetResponse], ResultWrapper[VersionGetResponse]),
        )

    async def graph(
        self,
        version_id: str,
        *,
        account_id: str,
        workflow_name: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> VersionGraphResponse:
        """
        Retrieves the graph visualization of a workflow version.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not workflow_name:
            raise ValueError(f"Expected a non-empty value for `workflow_name` but received {workflow_name!r}")
        if not version_id:
            raise ValueError(f"Expected a non-empty value for `version_id` but received {version_id!r}")
        return await self._get(
            path_template(
                "/accounts/{account_id}/workflows/{workflow_name}/versions/{version_id}/graph",
                account_id=account_id,
                workflow_name=workflow_name,
                version_id=version_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[VersionGraphResponse]._unwrapper,
            ),
            cast_to=cast(Type[VersionGraphResponse], ResultWrapper[VersionGraphResponse]),
        )


class VersionsResourceWithRawResponse:
    def __init__(self, versions: VersionsResource) -> None:
        self._versions = versions

        self.list = to_raw_response_wrapper(
            versions.list,
        )
        self.get = to_raw_response_wrapper(
            versions.get,
        )
        self.graph = to_raw_response_wrapper(
            versions.graph,
        )


class AsyncVersionsResourceWithRawResponse:
    def __init__(self, versions: AsyncVersionsResource) -> None:
        self._versions = versions

        self.list = async_to_raw_response_wrapper(
            versions.list,
        )
        self.get = async_to_raw_response_wrapper(
            versions.get,
        )
        self.graph = async_to_raw_response_wrapper(
            versions.graph,
        )


class VersionsResourceWithStreamingResponse:
    def __init__(self, versions: VersionsResource) -> None:
        self._versions = versions

        self.list = to_streamed_response_wrapper(
            versions.list,
        )
        self.get = to_streamed_response_wrapper(
            versions.get,
        )
        self.graph = to_streamed_response_wrapper(
            versions.graph,
        )


class AsyncVersionsResourceWithStreamingResponse:
    def __init__(self, versions: AsyncVersionsResource) -> None:
        self._versions = versions

        self.list = async_to_streamed_response_wrapper(
            versions.list,
        )
        self.get = async_to_streamed_response_wrapper(
            versions.get,
        )
        self.graph = async_to_streamed_response_wrapper(
            versions.graph,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict
from typing_extensions import Literal, Required, Annotated, TypedDict

from ..._utils import PropertyInfo

__all__ = [
    "RegistrationCreateParams",
    "Contacts",
    "ContactsAdministrator",
    "ContactsAdministratorPostalInfo",
    "ContactsAdministratorPostalInfoAddress",
    "ContactsBilling",
    "ContactsBillingPostalInfo",
    "ContactsBillingPostalInfoAddress",
    "ContactsRegistrant",
    "ContactsRegistrantPostalInfo",
    "ContactsRegistrantPostalInfoAddress",
    "ContactsTechnical",
    "ContactsTechnicalPostalInfo",
    "ContactsTechnicalPostalInfoAddress",
]


class RegistrationCreateParams(TypedDict, total=False):
    account_id: Required[str]
    """Identifier"""

    domain_name: Required[str]
    """
    Fully qualified domain name (FQDN) including the extension (e.g., `example.com`,
    `mybrand.app`). The domain name uniquely identifies a registration — the same
    domain cannot be registered twice, making it a natural idempotency key for
    registration requests.
    """

    acknowledgements: Dict[str, object]
    """
    User acknowledgements required by a specific extension or premium registration
    flow. The expected keys are described by the extension registration schema
    returned by the extension discovery endpoint.
    """

    auto_renew: bool
    """Enable or disable automatic renewal.

    Defaults to `false` if omitted. Setting this field to `true` is an explicit
    opt-in authorizing Cloudflare to charge the account's default payment method up
    to 30 days before domain expiry to renew the domain automatically. Renewal
    pricing may change over time based on registry pricing.
    """

    contact_extensions: Dict[str, object]
    """Registry-specific contact extension values for the registrant.

    The required keys and allowed values vary by extension and are described by
    `GET /accounts/{account_id}/registrar/extensions/{extension}` in the
    `registration_schema.properties.contact_extensions` object.

    Examples include `.us` nexus fields, `.uk` registrant type fields, and `.ca`
    legal type fields. Omit this object for extensions whose registration schema
    does not include `contact_extensions`.
    """

    contacts: Contacts
    """Contact data for the registration request.

    The per-extension schema returned by
    `GET /accounts/{account_id}/registrar/extensions/{extension}` is the
    authoritative contract for which contact roles are accepted. Every currently
    supported extension requires only `contacts.registrant` from API callers.
    Additional roles such as `technical`, `administrator`, and `billing` may be
    provided when the extension schema includes them. If a registry requires one of
    those roles and the caller omits it, Cloudflare may derive that contact from
    `contacts.registrant`.

    If the `contacts` object is omitted entirely from the request, or if
    `contacts.registrant` is not provided, the system will use the account's default
    address book entry as the registrant contact. This default must be
    pre-configured by the account owner at
    `https://dash.cloudflare.com/{account_id}/domains/registrations`, where they can
    create or update the address book entry and accept the required agreement. No
    API exists for managing address book entries at this time.

    If no default address book entry exists and no registrant contact is provided,
    the registration request will fail with a validation error.
    """

    privacy_mode: Literal["redaction"]
    """WHOIS privacy mode for the registration. Defaults to `redaction`.

    - `off`: Do not request WHOIS privacy.
    - `redaction`: Request WHOIS redaction where supported by the extension. Some
      extensions do not support privacy/redaction.
    """

    years: int
    """Number of years to register (1–10).

    If omitted, defaults to the minimum registration period required by the registry
    for this extension. For most extensions this is 1 year, but some extensions
    require longer minimum terms (e.g., `.ai` requires a minimum of 2 years).

    The registry for each extension may also enforce its own maximum registration
    term. If the requested value exceeds the registry's maximum, the registration
    will be rejected. When in doubt, use the default by omitting this field.
    """

    prefer: Annotated[str, PropertyInfo(alias="Prefer")]


class ContactsAdministratorPostalInfoAddress(TypedDict, total=False):
    """Physical mailing address for the registrant contact."""

    city: Required[str]
    """City or locality name."""

    country_code: Required[str]
    """Two-letter country code per ISO 3166-1 alpha-2 (e.g., `US`, `GB`, `CA`, `DE`)."""

    postal_code: Required[str]
    """Postal or ZIP code."""

    state: Required[str]
    """State, province, or region.

    Use the standard abbreviation where applicable (e.g., `TX` for Texas, `ON` for
    Ontario).
    """

    street: Required[str]
    """Street address including building/suite number."""


class ContactsAdministratorPostalInfo(TypedDict, total=False):
    """Postal/mailing information for the contact.

    The `name` field is the
    complete contact name in one string. Some registries require a complete
    personal name, including a family or last name where applicable, but this
    API does not accept separate first-name and last-name fields for
    registration contacts.
    """

    address: Required[ContactsAdministratorPostalInfoAddress]
    """Physical mailing address for the registrant contact."""

    name: Required[str]
    """
    Full legal name of the contact, including all required name components for an
    individual or authorized representative. Some registries require a complete
    personal name that includes a family or last name where applicable. Provide the
    complete name in this single field, for example `Ada Lovelace`; do not send
    separate first-name or last-name fields.
    """

    organization: str
    """Organization or company name. Optional for individual registrants."""


class ContactsAdministrator(TypedDict, total=False):
    """Contact data for the domain registration.

    This information
    is submitted to the domain registry and, depending on extension and
    privacy settings, may appear in public WHOIS records.
    """

    email: Required[str]
    """Email address for the registrant.

    Used for domain-related communications from the registry, including ownership
    verification and renewal notices.
    """

    phone: Required[str]
    """
    Phone number in E.164 format: `+{country_code}.{number}` with no spaces or
    dashes. Examples: `+1.5555555555` (US), `+44.2071234567` (UK), `+81.312345678`
    (Japan).
    """

    postal_info: Required[ContactsAdministratorPostalInfo]
    """Postal/mailing information for the contact.

    The `name` field is the complete contact name in one string. Some registries
    require a complete personal name, including a family or last name where
    applicable, but this API does not accept separate first-name and last-name
    fields for registration contacts.
    """

    fax: str
    """Fax number in E.164 format (e.g., `+1.5555555555`).

    Optional. Most registrations do not require a fax number.
    """


class ContactsBillingPostalInfoAddress(TypedDict, total=False):
    """Physical mailing address for the registrant contact."""

    city: Required[str]
    """City or locality name."""

    country_code: Required[str]
    """Two-letter country code per ISO 3166-1 alpha-2 (e.g., `US`, `GB`, `CA`, `DE`)."""

    postal_code: Required[str]
    """Postal or ZIP code."""

    state: Required[str]
    """State, province, or region.

    Use the standard abbreviation where applicable (e.g., `TX` for Texas, `ON` for
    Ontario).
    """

    street: Required[str]
    """Street address including building/suite number."""


class ContactsBillingPostalInfo(TypedDict, total=False):
    """Postal/mailing information for the contact.

    The `name` field is the
    complete contact name in one string. Some registries require a complete
    personal name, including a family or last name where applicable, but this
    API does not accept separate first-name and last-name fields for
    registration contacts.
    """

    address: Required[ContactsBillingPostalInfoAddress]
    """Physical mailing address for the registrant contact."""

    name: Required[str]
    """
    Full legal name of the contact, including all required name components for an
    individual or authorized representative. Some registries require a complete
    personal name that includes a family or last name where applicable. Provide the
    complete name in this single field, for example `Ada Lovelace`; do not send
    separate first-name or last-name fields.
    """

    organization: str
    """Organization or company name. Optional for individual registrants."""


class ContactsBilling(TypedDict, total=False):
    """Contact data for the domain registration.

    This information
    is submitted to the domain registry and, depending on extension and
    privacy settings, may appear in public WHOIS records.
    """

    email: Required[str]
    """Email address for the registrant.

    Used for domain-related communications from the registry, including ownership
    verification and renewal notices.
    """

    phone: Required[str]
    """
    Phone number in E.164 format: `+{country_code}.{number}` with no spaces or
    dashes. Examples: `+1.5555555555` (US), `+44.2071234567` (UK), `+81.312345678`
    (Japan).
    """

    postal_info: Required[ContactsBillingPostalInfo]
    """Postal/mailing information for the contact.

    The `name` field is the complete contact name in one string. Some registries
    require a complete personal name, including a family or last name where
    applicable, but this API does not accept separate first-name and last-name
    fields for registration contacts.
    """

    fax: str
    """Fax number in E.164 format (e.g., `+1.5555555555`).

    Optional. Most registrations do not require a fax number.
    """


class ContactsRegistrantPostalInfoAddress(TypedDict, total=False):
    """Physical mailing address for the registrant contact."""

    city: Required[str]
    """City or locality name."""

    country_code: Required[str]
    """Two-letter country code per ISO 3166-1 alpha-2 (e.g., `US`, `GB`, `CA`, `DE`)."""

    postal_code: Required[str]
    """Postal or ZIP code."""

    state: Required[str]
    """State, province, or region.

    Use the standard abbreviation where applicable (e.g., `TX` for Texas, `ON` for
    Ontario).
    """

    street: Required[str]
    """Street address including building/suite number."""


class ContactsRegistrantPostalInfo(TypedDict, total=False):
    """Postal/mailing information for the contact.

    The `name` field is the
    complete contact name in one string. Some registries require a complete
    personal name, including a family or last name where applicable, but this
    API does not accept separate first-name and last-name fields for
    registration contacts.
    """

    address: Required[ContactsRegistrantPostalInfoAddress]
    """Physical mailing address for the registrant contact."""

    name: Required[str]
    """
    Full legal name of the contact, including all required name components for an
    individual or authorized representative. Some registries require a complete
    personal name that includes a family or last name where applicable. Provide the
    complete name in this single field, for example `Ada Lovelace`; do not send
    separate first-name or last-name fields.
    """

    organization: str
    """Organization or company name. Optional for individual registrants."""


class ContactsRegistrant(TypedDict, total=False):
    """Contact data for the domain registration.

    This information
    is submitted to the domain registry and, depending on extension and
    privacy settings, may appear in public WHOIS records.
    """

    email: Required[str]
    """Email address for the registrant.

    Used for domain-related communications from the registry, including ownership
    verification and renewal notices.
    """

    phone: Required[str]
    """
    Phone number in E.164 format: `+{country_code}.{number}` with no spaces or
    dashes. Examples: `+1.5555555555` (US), `+44.2071234567` (UK), `+81.312345678`
    (Japan).
    """

    postal_info: Required[ContactsRegistrantPostalInfo]
    """Postal/mailing information for the contact.

    The `name` field is the complete contact name in one string. Some registries
    require a complete personal name, including a family or last name where
    applicable, but this API does not accept separate first-name and last-name
    fields for registration contacts.
    """

    fax: str
    """Fax number in E.164 format (e.g., `+1.5555555555`).

    Optional. Most registrations do not require a fax number.
    """


class ContactsTechnicalPostalInfoAddress(TypedDict, total=False):
    """Physical mailing address for the registrant contact."""

    city: Required[str]
    """City or locality name."""

    country_code: Required[str]
    """Two-letter country code per ISO 3166-1 alpha-2 (e.g., `US`, `GB`, `CA`, `DE`)."""

    postal_code: Required[str]
    """Postal or ZIP code."""

    state: Required[str]
    """State, province, or region.

    Use the standard abbreviation where applicable (e.g., `TX` for Texas, `ON` for
    Ontario).
    """

    street: Required[str]
    """Street address including building/suite number."""


class ContactsTechnicalPostalInfo(TypedDict, total=False):
    """Postal/mailing information for the contact.

    The `name` field is the
    complete contact name in one string. Some registries require a complete
    personal name, including a family or last name where applicable, but this
    API does not accept separate first-name and last-name fields for
    registration contacts.
    """

    address: Required[ContactsTechnicalPostalInfoAddress]
    """Physical mailing address for the registrant contact."""

    name: Required[str]
    """
    Full legal name of the contact, including all required name components for an
    individual or authorized representative. Some registries require a complete
    personal name that includes a family or last name where applicable. Provide the
    complete name in this single field, for example `Ada Lovelace`; do not send
    separate first-name or last-name fields.
    """

    organization: str
    """Organization or company name. Optional for individual registrants."""


class ContactsTechnical(TypedDict, total=False):
    """Contact data for the domain registration.

    This information
    is submitted to the domain registry and, depending on extension and
    privacy settings, may appear in public WHOIS records.
    """

    email: Required[str]
    """Email address for the registrant.

    Used for domain-related communications from the registry, including ownership
    verification and renewal notices.
    """

    phone: Required[str]
    """
    Phone number in E.164 format: `+{country_code}.{number}` with no spaces or
    dashes. Examples: `+1.5555555555` (US), `+44.2071234567` (UK), `+81.312345678`
    (Japan).
    """

    postal_info: Required[ContactsTechnicalPostalInfo]
    """Postal/mailing information for the contact.

    The `name` field is the complete contact name in one string. Some registries
    require a complete personal name, including a family or last name where
    applicable, but this API does not accept separate first-name and last-name
    fields for registration contacts.
    """

    fax: str
    """Fax number in E.164 format (e.g., `+1.5555555555`).

    Optional. Most registrations do not require a fax number.
    """


class Contacts(TypedDict, total=False):
    """Contact data for the registration request.

    The per-extension schema returned by
    `GET /accounts/{account_id}/registrar/extensions/{extension}` is the
    authoritative contract for which contact roles are accepted. Every
    currently supported extension requires only `contacts.registrant` from
    API callers. Additional roles such as `technical`, `administrator`, and
    `billing` may be provided when the extension schema includes them. If a
    registry requires one of those roles and the caller omits it, Cloudflare
    may derive that contact from `contacts.registrant`.

    If the `contacts` object is omitted entirely from the request, or if
    `contacts.registrant` is not provided, the system will use the account's
    default address book entry as the registrant contact. This default must be
    pre-configured by the account owner at
    `https://dash.cloudflare.com/{account_id}/domains/registrations`, where
    they can create or update the address book entry and accept the required
    agreement. No API exists for managing address book entries at this time.

    If no default address book entry exists and no registrant contact is
    provided, the registration request will fail with a validation error.
    """

    administrator: ContactsAdministrator
    """Contact data for the domain registration.

    This information is submitted to the domain registry and, depending on extension
    and privacy settings, may appear in public WHOIS records.
    """

    billing: ContactsBilling
    """Contact data for the domain registration.

    This information is submitted to the domain registry and, depending on extension
    and privacy settings, may appear in public WHOIS records.
    """

    registrant: ContactsRegistrant
    """Contact data for the domain registration.

    This information is submitted to the domain registry and, depending on extension
    and privacy settings, may appear in public WHOIS records.
    """

    technical: ContactsTechnical
    """Contact data for the domain registration.

    This information is submitted to the domain registry and, depending on extension
    and privacy settings, may appear in public WHOIS records.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, cast

import httpx

from ...._types import Body, Query, Headers, NotGiven, not_given
from ...._utils import path_template
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._wrappers import ResultWrapper
from ...._base_client import make_request_options
from ....types.cloudforce_one.threat_events.relate_delete_response import RelateDeleteResponse

__all__ = ["RelateResource", "AsyncRelateResource"]


class RelateResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> RelateResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return RelateResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> RelateResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return RelateResourceWithStreamingResponse(self)

    def delete(
        self,
        event_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> RelateDeleteResponse:
        """
        Remove one or more references from an event.

        Args:
          account_id: Account ID.

          event_id: Event UUID.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not event_id:
            raise ValueError(f"Expected a non-empty value for `event_id` but received {event_id!r}")
        return self._delete(
            path_template(
                "/accounts/{account_id}/cloudforce-one/events/relate/{event_id}",
                account_id=account_id,
                event_id=event_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[RelateDeleteResponse]._unwrapper,
            ),
            cast_to=cast(Type[RelateDeleteResponse], ResultWrapper[RelateDeleteResponse]),
        )


class AsyncRelateResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncRelateResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncRelateResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncRelateResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncRelateResourceWithStreamingResponse(self)

    async def delete(
        self,
        event_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> RelateDeleteResponse:
        """
        Remove one or more references from an event.

        Args:
          account_id: Account ID.

          event_id: Event UUID.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not event_id:
            raise ValueError(f"Expected a non-empty value for `event_id` but received {event_id!r}")
        return await self._delete(
            path_template(
                "/accounts/{account_id}/cloudforce-one/events/relate/{event_id}",
                account_id=account_id,
                event_id=event_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[RelateDeleteResponse]._unwrapper,
            ),
            cast_to=cast(Type[RelateDeleteResponse], ResultWrapper[RelateDeleteResponse]),
        )


class RelateResourceWithRawResponse:
    def __init__(self, relate: RelateResource) -> None:
        self._relate = relate

        self.delete = to_raw_response_wrapper(
            relate.delete,
        )


class AsyncRelateResourceWithRawResponse:
    def __init__(self, relate: AsyncRelateResource) -> None:
        self._relate = relate

        self.delete = async_to_raw_response_wrapper(
            relate.delete,
        )


class RelateResourceWithStreamingResponse:
    def __init__(self, relate: RelateResource) -> None:
        self._relate = relate

        self.delete = to_streamed_response_wrapper(
            relate.delete,
        )


class AsyncRelateResourceWithStreamingResponse:
    def __init__(self, relate: AsyncRelateResource) -> None:
        self._relate = relate

        self.delete = async_to_streamed_response_wrapper(
            relate.delete,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .jobs import (
    JobsResource,
    AsyncJobsResource,
    JobsResourceWithRawResponse,
    AsyncJobsResourceWithRawResponse,
    JobsResourceWithStreamingResponse,
    AsyncJobsResourceWithStreamingResponse,
)
from .webhooks import (
    WebhooksResource,
    AsyncWebhooksResource,
    WebhooksResourceWithRawResponse,
    AsyncWebhooksResourceWithRawResponse,
    WebhooksResourceWithStreamingResponse,
    AsyncWebhooksResourceWithStreamingResponse,
)

__all__ = [
    "JobsResource",
    "AsyncJobsResource",
    "JobsResourceWithRawResponse",
    "AsyncJobsResourceWithRawResponse",
    "JobsResourceWithStreamingResponse",
    "AsyncJobsResourceWithStreamingResponse",
    "WebhooksResource",
    "AsyncWebhooksResource",
    "WebhooksResourceWithRawResponse",
    "AsyncWebhooksResourceWithRawResponse",
    "WebhooksResourceWithStreamingResponse",
    "AsyncWebhooksResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["RobotGetParams"]


class RobotGetParams(TypedDict, total=False):
    zone_id: Required[str]

    subdomain: str
    """Optional subdomain to fetch robots.txt for.

    If omitted, fetches robots.txt for the zone apex domain.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["OwnershipValidateParams"]


class OwnershipValidateParams(TypedDict, total=False):
    account_id: Required[str]
    """Identifier."""

    destination_conf: Required[str]
    """The full URI for the bucket. This field only applies to `full` packet captures."""

    ownership_challenge: Required[str]
    """The ownership challenge filename stored in the bucket."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, cast

import httpx

from ...._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ...._utils import path_template, maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._wrappers import ResultWrapper
from ....pagination import SyncSinglePage, AsyncSinglePage
from ...._base_client import AsyncPaginator, make_request_options
from ....types.magic_transit.connectors import interrupt_create_params
from ....types.magic_transit.connectors.interrupt_list_response import InterruptListResponse
from ....types.magic_transit.connectors.interrupt_create_response import InterruptCreateResponse

__all__ = ["InterruptsResource", "AsyncInterruptsResource"]


class InterruptsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> InterruptsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return InterruptsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> InterruptsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return InterruptsResourceWithStreamingResponse(self)

    def create(
        self,
        connector_id: str,
        *,
        account_id: str,
        reboot: interrupt_create_params.Reboot | Omit = omit,
        restart: interrupt_create_params.Restart | Omit = omit,
        shutdown: interrupt_create_params.Shutdown | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> InterruptCreateResponse:
        """
        Creates an interrupt for a Magic WAN Connector.

        Args:
          account_id: Account identifier

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not connector_id:
            raise ValueError(f"Expected a non-empty value for `connector_id` but received {connector_id!r}")
        return self._post(
            path_template(
                "/accounts/{account_id}/magic/connectors/{connector_id}/interrupts",
                account_id=account_id,
                connector_id=connector_id,
            ),
            body=maybe_transform(
                {
                    "reboot": reboot,
                    "restart": restart,
                    "shutdown": shutdown,
                },
                interrupt_create_params.InterruptCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[InterruptCreateResponse]._unwrapper,
            ),
            cast_to=cast(Type[InterruptCreateResponse], ResultWrapper[InterruptCreateResponse]),
        )

    def list(
        self,
        connector_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncSinglePage[InterruptListResponse]:
        """
        Lists interrupts for a Magic WAN Connector.

        Args:
          account_id: Account identifier

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not connector_id:
            raise ValueError(f"Expected a non-empty value for `connector_id` but received {connector_id!r}")
        return self._get_api_list(
            path_template(
                "/accounts/{account_id}/magic/connectors/{connector_id}/interrupts",
                account_id=account_id,
                connector_id=connector_id,
            ),
            page=SyncSinglePage[InterruptListResponse],
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            model=InterruptListResponse,
        )


class AsyncInterruptsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncInterruptsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncInterruptsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncInterruptsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncInterruptsResourceWithStreamingResponse(self)

    async def create(
        self,
        connector_id: str,
        *,
        account_id: str,
        reboot: interrupt_create_params.Reboot | Omit = omit,
        restart: interrupt_create_params.Restart | Omit = omit,
        shutdown: interrupt_create_params.Shutdown | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> InterruptCreateResponse:
        """
        Creates an interrupt for a Magic WAN Connector.

        Args:
          account_id: Account identifier

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not connector_id:
            raise ValueError(f"Expected a non-empty value for `connector_id` but received {connector_id!r}")
        return await self._post(
            path_template(
                "/accounts/{account_id}/magic/connectors/{connector_id}/interrupts",
                account_id=account_id,
                connector_id=connector_id,
            ),
            body=await async_maybe_transform(
                {
                    "reboot": reboot,
                    "restart": restart,
                    "shutdown": shutdown,
                },
                interrupt_create_params.InterruptCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[InterruptCreateResponse]._unwrapper,
            ),
            cast_to=cast(Type[InterruptCreateResponse], ResultWrapper[InterruptCreateResponse]),
        )

    def list(
        self,
        connector_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[InterruptListResponse, AsyncSinglePage[InterruptListResponse]]:
        """
        Lists interrupts for a Magic WAN Connector.

        Args:
          account_id: Account identifier

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not connector_id:
            raise ValueError(f"Expected a non-empty value for `connector_id` but received {connector_id!r}")
        return self._get_api_list(
            path_template(
                "/accounts/{account_id}/magic/connectors/{connector_id}/interrupts",
                account_id=account_id,
                connector_id=connector_id,
            ),
            page=AsyncSinglePage[InterruptListResponse],
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            model=InterruptListResponse,
        )


class InterruptsResourceWithRawResponse:
    def __init__(self, interrupts: InterruptsResource) -> None:
        self._interrupts = interrupts

        self.create = to_raw_response_wrapper(
            interrupts.create,
        )
        self.list = to_raw_response_wrapper(
            interrupts.list,
        )


class AsyncInterruptsResourceWithRawResponse:
    def __init__(self, interrupts: AsyncInterruptsResource) -> None:
        self._interrupts = interrupts

        self.create = async_to_raw_response_wrapper(
            interrupts.create,
        )
        self.list = async_to_raw_response_wrapper(
            interrupts.list,
        )


class InterruptsResourceWithStreamingResponse:
    def __init__(self, interrupts: InterruptsResource) -> None:
        self._interrupts = interrupts

        self.create = to_streamed_response_wrapper(
            interrupts.create,
        )
        self.list = to_streamed_response_wrapper(
            interrupts.list,
        )


class AsyncInterruptsResourceWithStreamingResponse:
    def __init__(self, interrupts: AsyncInterruptsResource) -> None:
        self._interrupts = interrupts

        self.create = async_to_streamed_response_wrapper(
            interrupts.create,
        )
        self.list = async_to_streamed_response_wrapper(
            interrupts.list,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .tenants import (
    TenantsResource,
    AsyncTenantsResource,
    TenantsResourceWithRawResponse,
    AsyncTenantsResourceWithRawResponse,
    TenantsResourceWithStreamingResponse,
    AsyncTenantsResourceWithStreamingResponse,
)
from .accounts import (
    AccountsResource,
    AsyncAccountsResource,
    AccountsResourceWithRawResponse,
    AsyncAccountsResourceWithRawResponse,
    AccountsResourceWithStreamingResponse,
    AsyncAccountsResourceWithStreamingResponse,
)
from .memberships import (
    MembershipsResource,
    AsyncMembershipsResource,
    MembershipsResourceWithRawResponse,
    AsyncMembershipsResourceWithRawResponse,
    MembershipsResourceWithStreamingResponse,
    AsyncMembershipsResourceWithStreamingResponse,
)
from .entitlements import (
    EntitlementsResource,
    AsyncEntitlementsResource,
    EntitlementsResourceWithRawResponse,
    AsyncEntitlementsResourceWithRawResponse,
    EntitlementsResourceWithStreamingResponse,
    AsyncEntitlementsResourceWithStreamingResponse,
)
from .account_types import (
    AccountTypesResource,
    AsyncAccountTypesResource,
    AccountTypesResourceWithRawResponse,
    AsyncAccountTypesResourceWithRawResponse,
    AccountTypesResourceWithStreamingResponse,
    AsyncAccountTypesResourceWithStreamingResponse,
)

__all__ = [
    "AccountTypesResource",
    "AsyncAccountTypesResource",
    "AccountTypesResourceWithRawResponse",
    "AsyncAccountTypesResourceWithRawResponse",
    "AccountTypesResourceWithStreamingResponse",
    "AsyncAccountTypesResourceWithStreamingResponse",
    "AccountsResource",
    "AsyncAccountsResource",
    "AccountsResourceWithRawResponse",
    "AsyncAccountsResourceWithRawResponse",
    "AccountsResourceWithStreamingResponse",
    "AsyncAccountsResourceWithStreamingResponse",
    "EntitlementsResource",
    "AsyncEntitlementsResource",
    "EntitlementsResourceWithRawResponse",
    "AsyncEntitlementsResourceWithRawResponse",
    "EntitlementsResourceWithStreamingResponse",
    "AsyncEntitlementsResourceWithStreamingResponse",
    "MembershipsResource",
    "AsyncMembershipsResource",
    "MembershipsResourceWithRawResponse",
    "AsyncMembershipsResourceWithRawResponse",
    "MembershipsResourceWithStreamingResponse",
    "AsyncMembershipsResourceWithStreamingResponse",
    "TenantsResource",
    "AsyncTenantsResource",
    "TenantsResourceWithRawResponse",
    "AsyncTenantsResourceWithRawResponse",
    "TenantsResourceWithStreamingResponse",
    "AsyncTenantsResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, List, Optional
from datetime import datetime
from typing_extensions import Literal

from ....._models import BaseModel

__all__ = [
    "ContentListResponse",
    "DLPContext",
    "Integration",
    "IntegrationPolicy",
    "IntegrationVendor",
    "IntegrationZtEnrollment",
]


class DLPContext(BaseModel):
    """DLP context information for a finding."""

    created: datetime
    """When the DLP context was created."""

    entry_ids: List[str]
    """DLP Entry IDs."""

    profile_id: str
    """DLP Profile ID."""

    updated: datetime
    """When the DLP context was last updated."""

    id: Optional[str] = None
    """Unique identifier for the DLP context."""

    deleted: Optional[datetime] = None
    """When the DLP context was deleted."""

    match_context_max_extent: Optional[int] = None
    """DLP Right Boundary of match context."""

    match_context_min_extent: Optional[int] = None
    """DLP Left Boundary of match context."""

    match_context_payload: Optional[Dict[str, object]] = None
    """DLP Match context payload that matched the profile in question."""


class IntegrationPolicy(BaseModel):
    """Policy configuration for an integration."""

    id: Optional[str] = None
    """Policy identifier."""

    client_id: Optional[str] = None
    """OAuth client ID for the policy."""

    compliance_level: Optional[str] = None
    """Compliance level for the policy."""

    dlp_enabled: Optional[bool] = None
    """Whether DLP is enabled for this policy."""

    link: Optional[str] = None
    """Link to policy documentation."""

    name: Optional[str] = None
    """Policy name."""

    permissions: Optional[List[str]] = None
    """List of permissions included in the policy."""


class IntegrationVendor(BaseModel):
    """Information about a vendor/service provider."""

    id: str
    """The id of the vendor."""

    description: Optional[str] = None
    """Detailed information about what kinds of issues are detected for this vendor."""

    display_name: str
    """The display name of the vendor."""

    logo: str
    """Logo URL for the vendor."""

    name: str
    """The name of the vendor."""

    static_logo: str
    """Static logo URL for the vendor."""

    zt_enrollments: List[str]
    """The vendor's compatible Zero Trust products."""

    policies: Optional[List[Dict[str, object]]] = None
    """The policies related to the vendor."""


class IntegrationZtEnrollment(BaseModel):
    """Information about a Zero Trust product integration."""

    id: Optional[str] = None
    """The internal identifier of the Zero Trust Product."""

    description: Optional[str] = None
    """Brief description of the Zero Trust Product."""

    display_name: Optional[str] = None
    """The verbose name of the Zero Trust Product."""

    enabled: Optional[bool] = None
    """
    Flag to enable/disable access to the listed integration from the corresponding
    Cloudflare product.
    """


class Integration(BaseModel):
    """Summary information about an integration."""

    created: datetime
    """When entity was created."""

    last_hydrated: datetime
    """When were the integration credentials last updated."""

    name: str
    """Name of the integration."""

    permissions: List[str]
    """The vendor-specific permissions associated with the integration."""

    policy: IntegrationPolicy
    """Policy configuration for an integration."""

    status: str
    """Current status of the integration."""

    updated: datetime
    """Last entity was updated."""

    upgradable: bool
    """Whether the integrations permissions can be updated."""

    vendor: IntegrationVendor
    """Information about a vendor/service provider."""

    zt_enrollments: List[IntegrationZtEnrollment]
    """Zero Trust products associated with this integration."""

    id: Optional[str] = None
    """Integration ID."""

    credential_health_status: Optional[Literal["Initializing", "Healthy", "Unhealthy"]] = None
    """Health status of integration credentials."""

    credentials_expiry: Optional[datetime] = None
    """The date and time when the integration credentials will expire."""

    is_paused: Optional[bool] = None
    """Whether the given integration is paused by the user."""

    upgrade_dismissed: Optional[bool] = None
    """UI State as to whether a potential permissions upgrade has been dismissed."""


class ContentListResponse(BaseModel):
    """Content asset with DLP information."""

    asset_id: str
    """Unique identifier for the asset."""

    asset_name: str
    """Name of the asset."""

    dlp_contexts: List[DLPContext]
    """DLP context information for this asset."""

    dlp_profile_count: int
    """Number of DLP profiles that flagged this asset."""

    dlp_profile_ids: List[str]
    """IDs of DLP profiles that flagged this asset."""

    integration: Integration
    """Summary information about an integration."""

    latest_affliction_date: datetime
    """Most recent date this asset was flagged."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from ...._models import BaseModel

__all__ = ["TagListResponse", "Pagination", "Tag", "TagAlias", "TagExternalReference", "TagInternalAlias"]


class Pagination(BaseModel):
    page: float

    page_size: float = FieldInfo(alias="pageSize")

    total_count: float = FieldInfo(alias="totalCount")

    total_pages: float = FieldInfo(alias="totalPages")


class TagAlias(BaseModel):
    value: str

    confidence: Optional[int] = None

    tlp: Optional[Literal["red", "amber", "green", "white"]] = None


class TagExternalReference(BaseModel):
    url: str

    description: Optional[str] = None


class TagInternalAlias(BaseModel):
    value: str

    confidence: Optional[int] = None

    tlp: Optional[Literal["red", "amber", "green", "white"]] = None


class Tag(BaseModel):
    uuid: str

    value: str

    active_duration: Optional[str] = FieldInfo(alias="activeDuration", default=None)

    actor_category: Optional[str] = FieldInfo(alias="actorCategory", default=None)

    actor_category_confidence: Optional[int] = FieldInfo(alias="actorCategoryConfidence", default=None)
    """Confidence (1-10) in the actor variety (actorCategory).

    CFONE-only: stripped from responses to non-CFONE accounts.
    """

    aliases: Optional[List[TagAlias]] = None
    """Structured aliases ({ value, confidence 1-10, tlp }).

    CFONE-only: stripped from responses to non-CFONE accounts.
    """

    alias_group_names: Optional[List[str]] = FieldInfo(alias="aliasGroupNames", default=None)

    alias_group_names_internal: Optional[List[str]] = FieldInfo(alias="aliasGroupNamesInternal", default=None)

    analytic_priority: Optional[float] = FieldInfo(alias="analyticPriority", default=None)

    attribution_confidence: Optional[str] = FieldInfo(alias="attributionConfidence", default=None)

    attribution_confidence_score: Optional[int] = FieldInfo(alias="attributionConfidenceScore", default=None)

    attribution_organization: Optional[str] = FieldInfo(alias="attributionOrganization", default=None)

    category_name: Optional[str] = FieldInfo(alias="categoryName", default=None)

    category_uuid: Optional[str] = FieldInfo(alias="categoryUuid", default=None)

    date_of_discovery: Optional[str] = FieldInfo(alias="dateOfDiscovery", default=None)

    external_reference_links: Optional[List[str]] = FieldInfo(alias="externalReferenceLinks", default=None)

    external_references: Optional[List[TagExternalReference]] = FieldInfo(alias="externalReferences", default=None)
    """Structured external references ({ url, description }).

    Public: returned to all accounts.
    """

    internal_aliases: Optional[List[TagInternalAlias]] = FieldInfo(alias="internalAliases", default=None)
    """Internal structured aliases ({ value, confidence 1-10, tlp }).

    CFONE-only: never returned to non-CFONE accounts.
    """

    internal_description: Optional[str] = FieldInfo(alias="internalDescription", default=None)

    motive: Optional[str] = None

    motive_confidence: Optional[int] = FieldInfo(alias="motiveConfidence", default=None)
    """Confidence (1-10) in the actor motive.

    CFONE-only: stripped from responses to non-CFONE accounts.
    """

    opsec_level: Optional[str] = FieldInfo(alias="opsecLevel", default=None)

    origin_country_confidence: Optional[int] = FieldInfo(alias="originCountryConfidence", default=None)
    """Confidence (1-10) in the origin-country attribution.

    CFONE-only: stripped from responses to non-CFONE accounts.
    """

    origin_country_iso: Optional[str] = FieldInfo(alias="originCountryISO", default=None)

    origin_country_iso_alpha3: Optional[str] = FieldInfo(alias="originCountryISOAlpha3", default=None)

    origin_country_tlp: Optional[Literal["red", "amber", "green", "white"]] = FieldInfo(
        alias="originCountryTlp", default=None
    )
    """TLP marking for the origin-country attribution.

    CFONE-only: stripped from responses to non-CFONE accounts.
    """

    priority: Optional[float] = None

    sophistication_level: Optional[str] = FieldInfo(alias="sophisticationLevel", default=None)


class TagListResponse(BaseModel):
    pagination: Pagination

    tags: List[Tag]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, Optional, cast
from typing_extensions import Literal

import httpx

from .cancel import (
    CancelResource,
    AsyncCancelResource,
    CancelResourceWithRawResponse,
    AsyncCancelResourceWithRawResponse,
    CancelResourceWithStreamingResponse,
    AsyncCancelResourceWithStreamingResponse,
)
from .messages import (
    MessagesResource,
    AsyncMessagesResource,
    MessagesResourceWithRawResponse,
    AsyncMessagesResourceWithRawResponse,
    MessagesResourceWithStreamingResponse,
    AsyncMessagesResourceWithStreamingResponse,
)
from ....._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ....._utils import path_template, maybe_transform, async_maybe_transform
from ....._compat import cached_property
from ....._resource import SyncAPIResource, AsyncAPIResource
from ....._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ....._wrappers import ResultWrapper
from .....pagination import SyncV4PagePaginationArray, AsyncV4PagePaginationArray
from ....._base_client import AsyncPaginator, make_request_options
from .....types.email_security.investigate import bulk_list_params, bulk_create_params
from .....types.email_security.investigate.bulk_get_response import BulkGetResponse
from .....types.email_security.investigate.bulk_list_response import BulkListResponse
from .....types.email_security.investigate.bulk_create_response import BulkCreateResponse
from .....types.email_security.investigate.bulk_delete_response import BulkDeleteResponse

__all__ = ["BulkResource", "AsyncBulkResource"]


class BulkResource(SyncAPIResource):
    @cached_property
    def cancel(self) -> CancelResource:
        return CancelResource(self._client)

    @cached_property
    def messages(self) -> MessagesResource:
        return MessagesResource(self._client)

    @cached_property
    def with_raw_response(self) -> BulkResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return BulkResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> BulkResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return BulkResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        account_id: str,
        action: Literal["MOVE", "RELEASE"],
        search_params: bulk_create_params.SearchParams,
        comment: Optional[str] | Omit = omit,
        destination: Literal[
            "Inbox", "JunkEmail", "DeletedItems", "RecoverableItemsDeletions", "RecoverableItemsPurges"
        ]
        | Omit = omit,
        expected_disposition: Literal[
            "MALICIOUS",
            "MALICIOUS-BEC",
            "SUSPICIOUS",
            "SPOOF",
            "SPAM",
            "BULK",
            "ENCRYPTED",
            "EXTERNAL",
            "UNKNOWN",
            "NONE",
        ]
        | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> BulkCreateResponse:
        """
        Creates a new bulk action job to move or release messages that match the
        provided search parameters.

        Args:
          account_id: Identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._post(
            path_template("/accounts/{account_id}/email-security/investigate/bulk", account_id=account_id),
            body=maybe_transform(
                {
                    "action": action,
                    "search_params": search_params,
                    "comment": comment,
                    "destination": destination,
                    "expected_disposition": expected_disposition,
                },
                bulk_create_params.BulkCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[BulkCreateResponse]._unwrapper,
            ),
            cast_to=cast(Type[BulkCreateResponse], ResultWrapper[BulkCreateResponse]),
        )

    def list(
        self,
        *,
        account_id: str,
        action_type: Literal["MOVE", "RELEASE"] | Omit = omit,
        page: int | Omit = omit,
        per_page: int | Omit = omit,
        status: Literal["PENDING", "DISCOVERING", "PROCESSING", "COMPLETED", "FAILED", "CANCELLED", "SKIPPED"]
        | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncV4PagePaginationArray[BulkListResponse]:
        """
        Returns a paginated list of bulk action jobs for the account.

        Args:
          account_id: Identifier.

          page: Current page within paginated list of results.

          per_page: The number of results per page. Maximum value is 1000.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/email-security/investigate/bulk", account_id=account_id),
            page=SyncV4PagePaginationArray[BulkListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "action_type": action_type,
                        "page": page,
                        "per_page": per_page,
                        "status": status,
                    },
                    bulk_list_params.BulkListParams,
                ),
            ),
            model=BulkListResponse,
        )

    def delete(
        self,
        job_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> BulkDeleteResponse:
        """Deletes the job, removing it from all list and detail endpoints.

        Only jobs in a
        terminal state (`COMPLETED`, `CANCELLED`, `FAILED`, or `SKIPPED`) can be
        deleted. To stop an in-progress job without removing it, use the cancel endpoint
        instead.

        Args:
          account_id: Identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not job_id:
            raise ValueError(f"Expected a non-empty value for `job_id` but received {job_id!r}")
        return self._delete(
            path_template(
                "/accounts/{account_id}/email-security/investigate/bulk/{job_id}", account_id=account_id, job_id=job_id
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[BulkDeleteResponse]._unwrapper,
            ),
            cast_to=cast(Type[BulkDeleteResponse], ResultWrapper[BulkDeleteResponse]),
        )

    def get(
        self,
        job_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> BulkGetResponse:
        """
        Returns the status and details of a specific bulk action job.

        Args:
          account_id: Identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not job_id:
            raise ValueError(f"Expected a non-empty value for `job_id` but received {job_id!r}")
        return self._get(
            path_template(
                "/accounts/{account_id}/email-security/investigate/bulk/{job_id}", account_id=account_id, job_id=job_id
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[BulkGetResponse]._unwrapper,
            ),
            cast_to=cast(Type[BulkGetResponse], ResultWrapper[BulkGetResponse]),
        )


class AsyncBulkResource(AsyncAPIResource):
    @cached_property
    def cancel(self) -> AsyncCancelResource:
        return AsyncCancelResource(self._client)

    @cached_property
    def messages(self) -> AsyncMessagesResource:
        return AsyncMessagesResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncBulkResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncBulkResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncBulkResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncBulkResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        account_id: str,
        action: Literal["MOVE", "RELEASE"],
        search_params: bulk_create_params.SearchParams,
        comment: Optional[str] | Omit = omit,
        destination: Literal[
            "Inbox", "JunkEmail", "DeletedItems", "RecoverableItemsDeletions", "RecoverableItemsPurges"
        ]
        | Omit = omit,
        expected_disposition: Literal[
            "MALICIOUS",
            "MALICIOUS-BEC",
            "SUSPICIOUS",
            "SPOOF",
            "SPAM",
            "BULK",
            "ENCRYPTED",
            "EXTERNAL",
            "UNKNOWN",
            "NONE",
        ]
        | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> BulkCreateResponse:
        """
        Creates a new bulk action job to move or release messages that match the
        provided search parameters.

        Args:
          account_id: Identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._post(
            path_template("/accounts/{account_id}/email-security/investigate/bulk", account_id=account_id),
            body=await async_maybe_transform(
                {
                    "action": action,
                    "search_params": search_params,
                    "comment": comment,
                    "destination": destination,
                    "expected_disposition": expected_disposition,
                },
                bulk_create_params.BulkCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[BulkCreateResponse]._unwrapper,
            ),
            cast_to=cast(Type[BulkCreateResponse], ResultWrapper[BulkCreateResponse]),
        )

    def list(
        self,
        *,
        account_id: str,
        action_type: Literal["MOVE", "RELEASE"] | Omit = omit,
        page: int | Omit = omit,
        per_page: int | Omit = omit,
        status: Literal["PENDING", "DISCOVERING", "PROCESSING", "COMPLETED", "FAILED", "CANCELLED", "SKIPPED"]
        | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[BulkListResponse, AsyncV4PagePaginationArray[BulkListResponse]]:
        """
        Returns a paginated list of bulk action jobs for the account.

        Args:
          account_id: Identifier.

          page: Current page within paginated list of results.

          per_page: The number of results per page. Maximum value is 1000.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/email-security/investigate/bulk", account_id=account_id),
            page=AsyncV4PagePaginationArray[BulkListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "action_type": action_type,
                        "page": page,
                        "per_page": per_page,
                        "status": status,
                    },
                    bulk_list_params.BulkListParams,
                ),
            ),
            model=BulkListResponse,
        )

    async def delete(
        self,
        job_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> BulkDeleteResponse:
        """Deletes the job, removing it from all list and detail endpoints.

        Only jobs in a
        terminal state (`COMPLETED`, `CANCELLED`, `FAILED`, or `SKIPPED`) can be
        deleted. To stop an in-progress job without removing it, use the cancel endpoint
        instead.

        Args:
          account_id: Identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not job_id:
            raise ValueError(f"Expected a non-empty value for `job_id` but received {job_id!r}")
        return await self._delete(
            path_template(
                "/accounts/{account_id}/email-security/investigate/bulk/{job_id}", account_id=account_id, job_id=job_id
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[BulkDeleteResponse]._unwrapper,
            ),
            cast_to=cast(Type[BulkDeleteResponse], ResultWrapper[BulkDeleteResponse]),
        )

    async def get(
        self,
        job_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> BulkGetResponse:
        """
        Returns the status and details of a specific bulk action job.

        Args:
          account_id: Identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not job_id:
            raise ValueError(f"Expected a non-empty value for `job_id` but received {job_id!r}")
        return await self._get(
            path_template(
                "/accounts/{account_id}/email-security/investigate/bulk/{job_id}", account_id=account_id, job_id=job_id
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[BulkGetResponse]._unwrapper,
            ),
            cast_to=cast(Type[BulkGetResponse], ResultWrapper[BulkGetResponse]),
        )


class BulkResourceWithRawResponse:
    def __init__(self, bulk: BulkResource) -> None:
        self._bulk = bulk

        self.create = to_raw_response_wrapper(
            bulk.create,
        )
        self.list = to_raw_response_wrapper(
            bulk.list,
        )
        self.delete = to_raw_response_wrapper(
            bulk.delete,
        )
        self.get = to_raw_response_wrapper(
            bulk.get,
        )

    @cached_property
    def cancel(self) -> CancelResourceWithRawResponse:
        return CancelResourceWithRawResponse(self._bulk.cancel)

    @cached_property
    def messages(self) -> MessagesResourceWithRawResponse:
        return MessagesResourceWithRawResponse(self._bulk.messages)


class AsyncBulkResourceWithRawResponse:
    def __init__(self, bulk: AsyncBulkResource) -> None:
        self._bulk = bulk

        self.create = async_to_raw_response_wrapper(
            bulk.create,
        )
        self.list = async_to_raw_response_wrapper(
            bulk.list,
        )
        self.delete = async_to_raw_response_wrapper(
            bulk.delete,
        )
        self.get = async_to_raw_response_wrapper(
            bulk.get,
        )

    @cached_property
    def cancel(self) -> AsyncCancelResourceWithRawResponse:
        return AsyncCancelResourceWithRawResponse(self._bulk.cancel)

    @cached_property
    def messages(self) -> AsyncMessagesResourceWithRawResponse:
        return AsyncMessagesResourceWithRawResponse(self._bulk.messages)


class BulkResourceWithStreamingResponse:
    def __init__(self, bulk: BulkResource) -> None:
        self._bulk = bulk

        self.create = to_streamed_response_wrapper(
            bulk.create,
        )
        self.list = to_streamed_response_wrapper(
            bulk.list,
        )
        self.delete = to_streamed_response_wrapper(
            bulk.delete,
        )
        self.get = to_streamed_response_wrapper(
            bulk.get,
        )

    @cached_property
    def cancel(self) -> CancelResourceWithStreamingResponse:
        return CancelResourceWithStreamingResponse(self._bulk.cancel)

    @cached_property
    def messages(self) -> MessagesResourceWithStreamingResponse:
        return MessagesResourceWithStreamingResponse(self._bulk.messages)


class AsyncBulkResourceWithStreamingResponse:
    def __init__(self, bulk: AsyncBulkResource) -> None:
        self._bulk = bulk

        self.create = async_to_streamed_response_wrapper(
            bulk.create,
        )
        self.list = async_to_streamed_response_wrapper(
            bulk.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            bulk.delete,
        )
        self.get = async_to_streamed_response_wrapper(
            bulk.get,
        )

    @cached_property
    def cancel(self) -> AsyncCancelResourceWithStreamingResponse:
        return AsyncCancelResourceWithStreamingResponse(self._bulk.cancel)

    @cached_property
    def messages(self) -> AsyncMessagesResourceWithStreamingResponse:
        return AsyncMessagesResourceWithStreamingResponse(self._bulk.messages)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .job_list_params import JobListParams as JobListParams
from .job_create_params import JobCreateParams as JobCreateParams
from .job_export_params import JobExportParams as JobExportParams
from .job_list_response import JobListResponse as JobListResponse
from .job_create_response import JobCreateResponse as JobCreateResponse
from .job_export_response import JobExportResponse as JobExportResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["QueryGetParams"]


class QueryGetParams(TypedDict, total=False):
    account_id: Required[str]

    id: str

    page: int
    """Optional page number for paginated list requests.

    Defaults to 1 when only per_page is supplied. Omit page and per_page to preserve
    the legacy full-list response.
    """

    per_page: int
    """Optional number of queries per page for paginated list requests.

    Defaults to 100 when only page is supplied. Maximum 100. Omit page and per_page
    to preserve the legacy full-list response.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .type_list_params import TypeListParams as TypeListParams
from .type_list_response import TypeListResponse as TypeListResponse
from .aggregate_list_params import AggregateListParams as AggregateListParams
from .by_dataset_list_params import ByDatasetListParams as ByDatasetListParams
from .aggregate_list_response import AggregateListResponse as AggregateListResponse
from .by_dataset_get_response import ByDatasetGetResponse as ByDatasetGetResponse
from .by_dataset_list_response import ByDatasetListResponse as ByDatasetListResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from ...._models import BaseModel

__all__ = ["AlertingGetResponse"]


class AlertingGetResponse(BaseModel):
    """Certificate Transparency alerting subscription settings for a zone."""

    enabled: bool
    """Whether CT alerting is enabled for the zone."""

    emails: Optional[List[str]] = None
    """Email addresses that receive CT alert notifications.

    Only present and configurable for Business and Enterprise zones. Maximum of 10
    addresses. For Free and Pro zones, notifications are sent to all users with SSL
    permissions on the zone.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .findings import (
    FindingsResource,
    AsyncFindingsResource,
    FindingsResourceWithRawResponse,
    AsyncFindingsResourceWithRawResponse,
    FindingsResourceWithStreamingResponse,
    AsyncFindingsResourceWithStreamingResponse,
)
from .data_security import (
    DataSecurityResource,
    AsyncDataSecurityResource,
    DataSecurityResourceWithRawResponse,
    AsyncDataSecurityResourceWithRawResponse,
    DataSecurityResourceWithStreamingResponse,
    AsyncDataSecurityResourceWithStreamingResponse,
)
from .content_findings import (
    ContentFindingsResource,
    AsyncContentFindingsResource,
    ContentFindingsResourceWithRawResponse,
    AsyncContentFindingsResourceWithRawResponse,
    ContentFindingsResourceWithStreamingResponse,
    AsyncContentFindingsResourceWithStreamingResponse,
)

__all__ = [
    "ContentFindingsResource",
    "AsyncContentFindingsResource",
    "ContentFindingsResourceWithRawResponse",
    "AsyncContentFindingsResourceWithRawResponse",
    "ContentFindingsResourceWithStreamingResponse",
    "AsyncContentFindingsResourceWithStreamingResponse",
    "FindingsResource",
    "AsyncFindingsResource",
    "FindingsResourceWithRawResponse",
    "AsyncFindingsResourceWithRawResponse",
    "FindingsResourceWithStreamingResponse",
    "AsyncFindingsResourceWithStreamingResponse",
    "DataSecurityResource",
    "AsyncDataSecurityResource",
    "DataSecurityResourceWithRawResponse",
    "AsyncDataSecurityResourceWithRawResponse",
    "DataSecurityResourceWithStreamingResponse",
    "AsyncDataSecurityResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union, Iterable
from datetime import datetime
from typing_extensions import Literal

import httpx

from .types import (
    TypesResource,
    AsyncTypesResource,
    TypesResourceWithRawResponse,
    AsyncTypesResourceWithRawResponse,
    TypesResourceWithStreamingResponse,
    AsyncTypesResourceWithStreamingResponse,
)
from .aggregate import (
    AggregateResource,
    AsyncAggregateResource,
    AggregateResourceWithRawResponse,
    AsyncAggregateResourceWithRawResponse,
    AggregateResourceWithStreamingResponse,
    AsyncAggregateResourceWithStreamingResponse,
)
from ....._types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from ....._utils import path_template, maybe_transform, async_maybe_transform
from ....._compat import cached_property
from ....._resource import SyncAPIResource, AsyncAPIResource
from ....._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ....._base_client import make_request_options
from .by_dataset.by_dataset import (
    ByDatasetResource,
    AsyncByDatasetResource,
    ByDatasetResourceWithRawResponse,
    AsyncByDatasetResourceWithRawResponse,
    ByDatasetResourceWithStreamingResponse,
    AsyncByDatasetResourceWithStreamingResponse,
)
from .....types.cloudforce_one.threat_events import indicator_list_params
from .....types.cloudforce_one.threat_events.indicator_list_response import IndicatorListResponse

__all__ = ["IndicatorsResource", "AsyncIndicatorsResource"]


class IndicatorsResource(SyncAPIResource):
    @cached_property
    def aggregate(self) -> AggregateResource:
        return AggregateResource(self._client)

    @cached_property
    def types(self) -> TypesResource:
        return TypesResource(self._client)

    @cached_property
    def by_dataset(self) -> ByDatasetResource:
        return ByDatasetResource(self._client)

    @cached_property
    def with_raw_response(self) -> IndicatorsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return IndicatorsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> IndicatorsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return IndicatorsResourceWithStreamingResponse(self)

    def list(
        self,
        *,
        account_id: str,
        cache: Literal["from-graph"] | Omit = omit,
        created_after: Union[str, datetime] | Omit = omit,
        created_before: Union[str, datetime] | Omit = omit,
        dataset_ids: SequenceNotStr[str] | Omit = omit,
        format: Literal["json", "stix2", "taxii"] | Omit = omit,
        include_tags: bool | Omit = omit,
        include_total_count: bool | Omit = omit,
        indicator_type: str | Omit = omit,
        name: str | Omit = omit,
        page: float | Omit = omit,
        page_size: float | Omit = omit,
        related_events: SequenceNotStr[str] | Omit = omit,
        related_events_limit: float | Omit = omit,
        search: Iterable[indicator_list_params.Search] | Omit = omit,
        source: Literal["do", "r2catalog"] | Omit = omit,
        tags: SequenceNotStr[str] | Omit = omit,
        tag_search: Iterable[indicator_list_params.TagSearch] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> IndicatorListResponse:
        """Retrieves a paginated list of indicators across specified datasets.

        Use
        datasetIds=all or datasetIds=\\** to query all datasets for the account. If no
        datasetIds provided, uses the default dataset.

        Args:
          account_id: Account ID.

          cache: Cache strategy. 'from-graph' serves results from the graph-node KV cache when
              all requested UUIDs are cached; falls back to normal path on partial/zero hit.

          created_after: Filter indicators created on or after this date. Must use ISO 8601 format (e.g.,
              '2024-01-15T00:00:00Z').

          created_before: Filter indicators created on or before this date. Must use ISO 8601 format
              (e.g., '2024-12-31T23:59:59Z').

          dataset_ids: Dataset IDs to query indicators from (array of UUIDs), or special value 'all' or
              '\\**' to query all datasets. If not provided, uses the default dataset.

          format: Output format for indicator data. 'json' returns the default format, 'stix2'
              returns STIX 2.1 Indicator SDOs, 'taxii' returns a TAXII 2.1 Envelope with
              Content-Type application/taxii+json;version=2.1.

          include_tags: Whether to include full tag details for each indicator. Defaults to true.

          include_total_count: Whether to compute accurate total count via COUNT(\\**). Defaults to false for
              performance. When false, total_count is an approximation.

          name: Filter indicators by value using substring match (LIKE). Legacy alternative to
              structured search.

          related_events: Filter by related event IDs

          related_events_limit: Limit the number of related events returned per indicator. Default: 2. Set to 0
              for none, -1 for all events.

          search: Structured search as a JSON array of {field, op, value} objects. Searchable
              fields: value, indicatorType, uuid. Supports operators: equals, not, contains,
              startsWith, endsWith, gt, lt, gte, lte, like, in, find. Use the 'in' operator
              with an array value to bulk-check up to 100 indicators in a single request, e.g.
              search=[{"field":"value","op":"in","value":["evil.com","bad.org"]}]. Multiple
              conditions are AND'd together. Max 10 conditions per request.

          source: Read backend. 'do' (default) reads Durable Object storage. 'r2catalog' reads R2
              Data Catalog (admin-only, experimental; supports a subset of search fields).

          tags: Filter by tag values or UUIDs. Indicators must have at least one of the
              specified tags (OR logic). Supports both tag UUID and tag value.

          tag_search: Structured tag-metadata filter as a JSON array of {field, op, value} objects.
              Operates against the per-dataset IndicatorTag mirror so you can find indicators
              by tag attributes (origin country, motive, sophistication, priority, etc.)
              without a separate Tags lookup. Common dashboard usage: drill from a country
              into indicators, e.g.
              tagSearch=[{"field":"originCountryISO","op":"in","value":["IR","CN"]}]. Country
              values may be passed as alpha-2, alpha-3, name, or alias (e.g. "iran").
              Operators: equals, not, gt/gte/lt/lte (numeric only),
              contains/like/find/startsWith/endsWith (string only), in. AND-joined across
              entries; combined with `tags`, a matching tag must satisfy both. Max 10 entries
              per request, max 100 values per 'in'. Performance notes: `originCountryISO` uses
              its B-tree index for equals/not/in. `priority` uses its B-tree index for numeric
              comparisons. Other string columns (`actorCategory`, `motive`, etc.) are
              case-insensitive and unindexed; current catalog size makes this a non-issue.
              `endsWith` and `aliasGroupNames` contains/like are leading-wildcard scans and
              slow on large result sets. `aliasGroupNames` matches on the JSON-encoded text,
              so substrings can cross alias boundaries ("apt28" also matches "apt280" when
              both appear in the same tag's alias list).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get(
            path_template("/accounts/{account_id}/cloudforce-one/events/indicators", account_id=account_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "cache": cache,
                        "created_after": created_after,
                        "created_before": created_before,
                        "dataset_ids": dataset_ids,
                        "format": format,
                        "include_tags": include_tags,
                        "include_total_count": include_total_count,
                        "indicator_type": indicator_type,
                        "name": name,
                        "page": page,
                        "page_size": page_size,
                        "related_events": related_events,
                        "related_events_limit": related_events_limit,
                        "search": search,
                        "source": source,
                        "tags": tags,
                        "tag_search": tag_search,
                    },
                    indicator_list_params.IndicatorListParams,
                ),
            ),
            cast_to=IndicatorListResponse,
        )


class AsyncIndicatorsResource(AsyncAPIResource):
    @cached_property
    def aggregate(self) -> AsyncAggregateResource:
        return AsyncAggregateResource(self._client)

    @cached_property
    def types(self) -> AsyncTypesResource:
        return AsyncTypesResource(self._client)

    @cached_property
    def by_dataset(self) -> AsyncByDatasetResource:
        return AsyncByDatasetResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncIndicatorsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncIndicatorsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncIndicatorsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncIndicatorsResourceWithStreamingResponse(self)

    async def list(
        self,
        *,
        account_id: str,
        cache: Literal["from-graph"] | Omit = omit,
        created_after: Union[str, datetime] | Omit = omit,
        created_before: Union[str, datetime] | Omit = omit,
        dataset_ids: SequenceNotStr[str] | Omit = omit,
        format: Literal["json", "stix2", "taxii"] | Omit = omit,
        include_tags: bool | Omit = omit,
        include_total_count: bool | Omit = omit,
        indicator_type: str | Omit = omit,
        name: str | Omit = omit,
        page: float | Omit = omit,
        page_size: float | Omit = omit,
        related_events: SequenceNotStr[str] | Omit = omit,
        related_events_limit: float | Omit = omit,
        search: Iterable[indicator_list_params.Search] | Omit = omit,
        source: Literal["do", "r2catalog"] | Omit = omit,
        tags: SequenceNotStr[str] | Omit = omit,
        tag_search: Iterable[indicator_list_params.TagSearch] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> IndicatorListResponse:
        """Retrieves a paginated list of indicators across specified datasets.

        Use
        datasetIds=all or datasetIds=\\** to query all datasets for the account. If no
        datasetIds provided, uses the default dataset.

        Args:
          account_id: Account ID.

          cache: Cache strategy. 'from-graph' serves results from the graph-node KV cache when
              all requested UUIDs are cached; falls back to normal path on partial/zero hit.

          created_after: Filter indicators created on or after this date. Must use ISO 8601 format (e.g.,
              '2024-01-15T00:00:00Z').

          created_before: Filter indicators created on or before this date. Must use ISO 8601 format
              (e.g., '2024-12-31T23:59:59Z').

          dataset_ids: Dataset IDs to query indicators from (array of UUIDs), or special value 'all' or
              '\\**' to query all datasets. If not provided, uses the default dataset.

          format: Output format for indicator data. 'json' returns the default format, 'stix2'
              returns STIX 2.1 Indicator SDOs, 'taxii' returns a TAXII 2.1 Envelope with
              Content-Type application/taxii+json;version=2.1.

          include_tags: Whether to include full tag details for each indicator. Defaults to true.

          include_total_count: Whether to compute accurate total count via COUNT(\\**). Defaults to false for
              performance. When false, total_count is an approximation.

          name: Filter indicators by value using substring match (LIKE). Legacy alternative to
              structured search.

          related_events: Filter by related event IDs

          related_events_limit: Limit the number of related events returned per indicator. Default: 2. Set to 0
              for none, -1 for all events.

          search: Structured search as a JSON array of {field, op, value} objects. Searchable
              fields: value, indicatorType, uuid. Supports operators: equals, not, contains,
              startsWith, endsWith, gt, lt, gte, lte, like, in, find. Use the 'in' operator
              with an array value to bulk-check up to 100 indicators in a single request, e.g.
              search=[{"field":"value","op":"in","value":["evil.com","bad.org"]}]. Multiple
              conditions are AND'd together. Max 10 conditions per request.

          source: Read backend. 'do' (default) reads Durable Object storage. 'r2catalog' reads R2
              Data Catalog (admin-only, experimental; supports a subset of search fields).

          tags: Filter by tag values or UUIDs. Indicators must have at least one of the
              specified tags (OR logic). Supports both tag UUID and tag value.

          tag_search: Structured tag-metadata filter as a JSON array of {field, op, value} objects.
              Operates against the per-dataset IndicatorTag mirror so you can find indicators
              by tag attributes (origin country, motive, sophistication, priority, etc.)
              without a separate Tags lookup. Common dashboard usage: drill from a country
              into indicators, e.g.
              tagSearch=[{"field":"originCountryISO","op":"in","value":["IR","CN"]}]. Country
              values may be passed as alpha-2, alpha-3, name, or alias (e.g. "iran").
              Operators: equals, not, gt/gte/lt/lte (numeric only),
              contains/like/find/startsWith/endsWith (string only), in. AND-joined across
              entries; combined with `tags`, a matching tag must satisfy both. Max 10 entries
              per request, max 100 values per 'in'. Performance notes: `originCountryISO` uses
              its B-tree index for equals/not/in. `priority` uses its B-tree index for numeric
              comparisons. Other string columns (`actorCategory`, `motive`, etc.) are
              case-insensitive and unindexed; current catalog size makes this a non-issue.
              `endsWith` and `aliasGroupNames` contains/like are leading-wildcard scans and
              slow on large result sets. `aliasGroupNames` matches on the JSON-encoded text,
              so substrings can cross alias boundaries ("apt28" also matches "apt280" when
              both appear in the same tag's alias list).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._get(
            path_template("/accounts/{account_id}/cloudforce-one/events/indicators", account_id=account_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "cache": cache,
                        "created_after": created_after,
                        "created_before": created_before,
                        "dataset_ids": dataset_ids,
                        "format": format,
                        "include_tags": include_tags,
                        "include_total_count": include_total_count,
                        "indicator_type": indicator_type,
                        "name": name,
                        "page": page,
                        "page_size": page_size,
                        "related_events": related_events,
                        "related_events_limit": related_events_limit,
                        "search": search,
                        "source": source,
                        "tags": tags,
                        "tag_search": tag_search,
                    },
                    indicator_list_params.IndicatorListParams,
                ),
            ),
            cast_to=IndicatorListResponse,
        )


class IndicatorsResourceWithRawResponse:
    def __init__(self, indicators: IndicatorsResource) -> None:
        self._indicators = indicators

        self.list = to_raw_response_wrapper(
            indicators.list,
        )

    @cached_property
    def aggregate(self) -> AggregateResourceWithRawResponse:
        return AggregateResourceWithRawResponse(self._indicators.aggregate)

    @cached_property
    def types(self) -> TypesResourceWithRawResponse:
        return TypesResourceWithRawResponse(self._indicators.types)

    @cached_property
    def by_dataset(self) -> ByDatasetResourceWithRawResponse:
        return ByDatasetResourceWithRawResponse(self._indicators.by_dataset)


class AsyncIndicatorsResourceWithRawResponse:
    def __init__(self, indicators: AsyncIndicatorsResource) -> None:
        self._indicators = indicators

        self.list = async_to_raw_response_wrapper(
            indicators.list,
        )

    @cached_property
    def aggregate(self) -> AsyncAggregateResourceWithRawResponse:
        return AsyncAggregateResourceWithRawResponse(self._indicators.aggregate)

    @cached_property
    def types(self) -> AsyncTypesResourceWithRawResponse:
        return AsyncTypesResourceWithRawResponse(self._indicators.types)

    @cached_property
    def by_dataset(self) -> AsyncByDatasetResourceWithRawResponse:
        return AsyncByDatasetResourceWithRawResponse(self._indicators.by_dataset)


class IndicatorsResourceWithStreamingResponse:
    def __init__(self, indicators: IndicatorsResource) -> None:
        self._indicators = indicators

        self.list = to_streamed_response_wrapper(
            indicators.list,
        )

    @cached_property
    def aggregate(self) -> AggregateResourceWithStreamingResponse:
        return AggregateResourceWithStreamingResponse(self._indicators.aggregate)

    @cached_property
    def types(self) -> TypesResourceWithStreamingResponse:
        return TypesResourceWithStreamingResponse(self._indicators.types)

    @cached_property
    def by_dataset(self) -> ByDatasetResourceWithStreamingResponse:
        return ByDatasetResourceWithStreamingResponse(self._indicators.by_dataset)


class AsyncIndicatorsResourceWithStreamingResponse:
    def __init__(self, indicators: AsyncIndicatorsResource) -> None:
        self._indicators = indicators

        self.list = async_to_streamed_response_wrapper(
            indicators.list,
        )

    @cached_property
    def aggregate(self) -> AsyncAggregateResourceWithStreamingResponse:
        return AsyncAggregateResourceWithStreamingResponse(self._indicators.aggregate)

    @cached_property
    def types(self) -> AsyncTypesResourceWithStreamingResponse:
        return AsyncTypesResourceWithStreamingResponse(self._indicators.types)

    @cached_property
    def by_dataset(self) -> AsyncByDatasetResourceWithStreamingResponse:
        return AsyncByDatasetResourceWithStreamingResponse(self._indicators.by_dataset)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = [
    "RelayCreateResponse",
    "Config",
    "ConfigUpstreams",
    "ConfigUpstreamsUpstream",
    "Issuer",
    "IssuerCloudflareToken",
]


class ConfigUpstreamsUpstream(BaseModel):
    """A single upstream MOQT server publisher."""

    url: str
    """Upstream MOQT server publisher URL.

    Must be an absolute URL with a host and a scheme crique can dial: moqt:// (raw
    QUIC) or https:// (WebTransport). Validated on update (PUT); rejected
    with 21013.
    """


class ConfigUpstreams(BaseModel):
    """
    Upstreams are external MOQT server publishers that a relay falls back
    to when it has no local publisher for a requested namespace/track.
    """

    enabled: Optional[bool] = None

    upstreams: Optional[List[ConfigUpstreamsUpstream]] = None
    """Ordered list of upstream MOQT server publishers.

    Each entry is an object (not a bare string) so per-upstream configuration can be
    added in the future without another breaking change.
    """


class Config(BaseModel):
    upstreams: Optional[ConfigUpstreams] = None
    """
    Upstreams are external MOQT server publishers that a relay falls back to when it
    has no local publisher for a requested namespace/track.
    """


class IssuerCloudflareToken(BaseModel):
    created: datetime

    expires: datetime
    """Mandatory; no more than 1 year after `created`."""

    jti: str
    """Token identity and registry key (32 hex chars)."""

    operations: List[Literal["publish", "subscribe"]]
    """Signed allowlist of what the token may do.

    V1 coarse roles; the array form extends to fine-grained MoQT message names later
    without a breaking change.
    """

    label: Optional[str] = None
    """Optional, customer-set."""

    secret: Optional[str] = None
    """The signed JWT.

    Present ONLY in create / auto-create responses (shown once); never returned by
    list, never stored.
    """


class Issuer(BaseModel):
    """One arm of the discriminated-union token collection."""

    cloudflare_tokens: List[IssuerCloudflareToken]
    """Always present ([] when empty)."""

    issuer: Literal["cloudflare"]

    type: Literal["cloudflare_jwt"]


class RelayCreateResponse(BaseModel):
    """
    Relay with its auto-created default token pair (one full-access
    [publish, subscribe] and one [subscribe]-only), each with its one-time
    secret, wrapped in the issuers envelope.
    """

    config: Config

    created: datetime

    issuers: List[Issuer]
    """Token collection (discriminated union on `type`).

    On create this holds the auto-created default pair, each including its one-time
    secret.
    """

    modified: datetime

    name: str

    uid: str
    """Server-generated unique identifier (32 hex chars)."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .dmarc_report_edit_params import DMARCReportEditParams as DMARCReportEditParams
from .dmarc_report_get_response import DMARCReportGetResponse as DMARCReportGetResponse
from .dmarc_report_edit_response import DMARCReportEditResponse as DMARCReportEditResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["FindingTuneSeverityParams"]


class FindingTuneSeverityParams(TypedDict, total=False):
    account_id: Required[str]

    new_severity: Required[Literal[1, 2, 3, 4]]
    """The numeric severity value to apply to the finding."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .catalog import (
    CatalogResource,
    AsyncCatalogResource,
    CatalogResourceWithRawResponse,
    AsyncCatalogResourceWithRawResponse,
    CatalogResourceWithStreamingResponse,
    AsyncCatalogResourceWithStreamingResponse,
)
from .by_dataset import (
    ByDatasetResource,
    AsyncByDatasetResource,
    ByDatasetResourceWithRawResponse,
    AsyncByDatasetResourceWithRawResponse,
    ByDatasetResourceWithStreamingResponse,
    AsyncByDatasetResourceWithStreamingResponse,
)
from .target_industries import (
    TargetIndustriesResource,
    AsyncTargetIndustriesResource,
    TargetIndustriesResourceWithRawResponse,
    AsyncTargetIndustriesResourceWithRawResponse,
    TargetIndustriesResourceWithStreamingResponse,
    AsyncTargetIndustriesResourceWithStreamingResponse,
)

__all__ = [
    "ByDatasetResource",
    "AsyncByDatasetResource",
    "ByDatasetResourceWithRawResponse",
    "AsyncByDatasetResourceWithRawResponse",
    "ByDatasetResourceWithStreamingResponse",
    "AsyncByDatasetResourceWithStreamingResponse",
    "CatalogResource",
    "AsyncCatalogResource",
    "CatalogResourceWithRawResponse",
    "AsyncCatalogResourceWithRawResponse",
    "CatalogResourceWithStreamingResponse",
    "AsyncCatalogResourceWithStreamingResponse",
    "TargetIndustriesResource",
    "AsyncTargetIndustriesResource",
    "TargetIndustriesResourceWithRawResponse",
    "AsyncTargetIndustriesResourceWithRawResponse",
    "TargetIndustriesResourceWithStreamingResponse",
    "AsyncTargetIndustriesResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["ApplicationListParams"]


class ApplicationListParams(TypedDict, total=False):
    account_id: Required[str]

    environment: str
    """Filter by supported environment (standard, fedramp)."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["CustomProviderCreateParams"]


class CustomProviderCreateParams(TypedDict, total=False):
    account_id: Required[str]

    base_url: Required[str]

    name: Required[str]

    slug: Required[str]

    beta: bool

    curl_example: str

    description: str

    enable: bool

    headers: str

    js_example: str

    link: str

    position: int



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .finding_summary_params import FindingSummaryParams as FindingSummaryParams
from .finding_summary_response import FindingSummaryResponse as FindingSummaryResponse
from .finding_timeseries_params import FindingTimeseriesParams as FindingTimeseriesParams
from .finding_timeseries_response import FindingTimeseriesResponse as FindingTimeseriesResponse
from .content_finding_top_n_params import ContentFindingTopNParams as ContentFindingTopNParams
from .content_finding_top_n_response import ContentFindingTopNResponse as ContentFindingTopNResponse
